/*
 * sysstat: System performance tools for Linux
 * (C) 1999-2018 by Sebastien Godard (sysstat <at> orange.fr)
 */

#ifndef _SYSCONFIG_H
#define _SYSCONFIG_H

/* sysstat configuration directory */
#define IOCONF		"/etc/sysconfig/sysstat.ioconf"
#define LOCAL_IOCONF	"./sysstat.ioconf"

#endif  /* _SYSCONFIG_H */
